package net.wanji.opt.service.impl;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.gts.GeometryUtil;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.mapper.BaseAreaInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.opt.bo.AddOrUpdateAreaBO;
import net.wanji.opt.bo.PolygonBO;
import net.wanji.opt.service.CrossManageService;
import net.wanji.opt.vo.CrossIdAndNameVO;
import org.locationtech.jts.geom.Geometry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/6/25 8:56
 */
@Service
@Slf4j
public class CrossManageServiceImpl implements CrossManageService {
    private final BaseCrossInfoMapper crossInfoMapper;
    private final BaseAreaInfoMapper baseAreaInfoMapper;

    public CrossManageServiceImpl(BaseCrossInfoMapper crossInfoMapper, BaseAreaInfoMapper baseAreaInfoMapper) {
        this.crossInfoMapper = crossInfoMapper;
        this.baseAreaInfoMapper = baseAreaInfoMapper;
    }

    @Override
    public List<CrossIdAndNameVO> selectCrossesByPolygon(PolygonBO polygonBO) {
        List<CrossIdAndNameVO> res = new ArrayList<>();
        String wkt = polygonBO.getWkt();
        List<BaseCrossInfoPO> crossList = crossInfoMapper.selectAll();

        for (BaseCrossInfoPO cross : crossList) {
            String location = cross.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String crossLonLat = lonLat[0] + "," + lonLat[1];
            Geometry wktGeo = GeometryUtil.str2Geometry(wkt);
            Geometry crossGeo = GeometryUtil.str2Geometry(crossLonLat);
            // 判断点和多边形是否相交
            boolean isIntersect = wktGeo.intersects(crossGeo);
            if (isIntersect) {
                CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
                crossIdAndNameVO.setCrossId(cross.getId());
                crossIdAndNameVO.setCrossName(cross.getName());
                res.add(crossIdAndNameVO);
            }
        }

        return res;
    }

    @Override
    @Transactional
    public void addOrUpdateArea(AddOrUpdateAreaBO addOrUpdateAreaBO) {
        Integer areaId = addOrUpdateAreaBO.getAreaId();
        String areaName = addOrUpdateAreaBO.getAreaName();
        String remark = addOrUpdateAreaBO.getRemark();
        List<String> crossIdList = addOrUpdateAreaBO.getCrossIdList();

        if (ObjectUtil.isNotEmpty(areaId)) {
            // 修改
            // 修改子区名称和备注
            baseAreaInfoMapper.updateAreaNameAndRemark(areaName, remark, areaId);
            // 修改子区关联路口

        } else {
            // 新增
        }
    }
}
